import os
from Datapyck.errors import *

class Datapack:
  def __init__(self, name="New Datapack", description="A datapack created using the Datapyck Python module.", version="1.19", directory=""):
    if type(name) is not str:
      raise ArgTypeError("Datapack", name, 1, str)
    if type(description) is not str:
      raise ArgTypeError("Datapack", description, 2, str)
    if type(version) is not str:
      raise ArgTypeError("Datapack", version, 3, str)
    if type(directory) is not str:
      raise ArgTypeError("Datapack", directory, 4, str)
    if version not in ["1.17", "1.17.1", "1.18", "1.18.1", "1.18.2", "1.19", "1.19.1", "1.19.2", "1.19.3"]:
      raise ValueError(f"not supported version \'{version}\'")
    if not os.path.exists(directory) and directory != "":
      raise FileNotFoundError(f"no such directory: \'{directory}\'")
    self.name = name
    self.description = description
    self.version = version
    self.directory = directory
    self.namespace = name.lower().replace(" ", "_")
    #Make the datapack
    def checkpath(name):
      if os.path.exists(os.path.join(self.directory, name)):
        try:
          os.rename(os.path.join(self.directory, name), name + "_original")
        except:
          checkpath(name + "_original")
    def pathtry():
      try:
        os.mkdir(os.path.join(self.directory, self.name))
      except:
        checkpath(self.name)
        pathtry()
    pathtry()
    pack = open(os.path.join(self.directory, self.name, "pack.mcmeta"), "w")
    if self.version.startswith("1.17"):
      pack.write("{\n  \"pack\": {\n    \"pack_format\": 7,\n    \"description\": \"" + description + "\"\n  }\n}")
    if self.version.startswith("1.18") and not version == "1.18.2":
      pack.write("{\n  \"pack\": {\n    \"pack_format\": 8,\n    \"description\": \"" + description + "\"\n  }\n}")
    if self.version == "1.18.2":
      pack.write("{\n  \"pack\": {\n    \"pack_format\": 9,\n    \"description\": \"" + description + "\"\n  }\n}")
    if self.version.startswith("1.19"):
      pack.write("{\n  \"pack\": {\n    \"pack_format\": 10,\n    \"description\": \"" + description + "\"\n  }\n}")
    os.mkdir(os.path.join(self.directory, self.name, "data"))
    os.mkdir(os.path.join(self.directory, self.name, "data", "minecraft"))
    os.mkdir(os.path.join(self.directory, self.name, "data", "minecraft", "tags"))
    os.mkdir(os.path.join(self.directory, self.name, "data", "minecraft", "tags", "functions"))
    loadjson = open(os.path.join(self.directory, self.name, "data", "minecraft", "tags", "functions", "load.json"), "w")
    loadjson.write("{\n	\"values\": [\n		\"" + self.namespace + ":load\"\n	]\n}")
    tickjson = open(os.path.join(self.directory, self.name, "data", "minecraft", "tags", "functions", "tick.json"), "w")
    tickjson.write("{\n	\"values\": [\n		\"" + self.namespace + ":tick\"\n	]\n}")
    os.mkdir(os.path.join(self.directory, self.name, "data", self.namespace))
    os.mkdir(os.path.join(self.directory, self.name, "data", self.namespace, "functions"))
    open(os.path.join(self.directory, self.name, "data", self.namespace, "functions", "load.mcfunction"), "x")
    open(os.path.join(self.directory, self.name, "data", self.namespace, "functions", "tick.mcfunction"), "x")
  def write(self, function, command):
    if type(function) is not str:
      raise ArgTypeError("write", function, 1, str)
    file = open(os.path.join(self.directory, self.name, "data", self.namespace, "functions", f"{function}.mcfunction"), "a")
    file.write(str(command) + "\n")


def ArgTypeError(function, argument, pos, *maintypes):
    if type(function) is not str:
        raise TypeError(f"ArgTypeError expected str, not {repr(type(function))[8:-2]} (pos 1)")
    if type(pos) is not int:
        raise TypeError(f"ArgTypeError expected int, not {repr(type(function))[8:-2]} (pos 3)")
    length = len(maintypes)
    if length == 1:
        return TypeError(f"{function} expected {repr(maintypes[0])[8:-2]}, not {repr(type(argument))[8:-2]} (pos {str(pos)})")
    elif length > 1:
        error = f"{function} expected "
        for value, maintype in enumerate(maintypes):
            if value == 0:
                error += f"{repr(maintype)[8:-2]}"
            elif value + 1 == length:
                if length == 2:
                    error += f" or {repr(maintype)[8:-2]}"
                else:
                    error += f", or {repr(maintype)[8:-2]}"
            else:
                error += f", {repr(maintype)[8:-2]}"
        error += f", not {repr(type(argument))[8:-2]} (pos {str(pos)})"
        return TypeError(error)
    else:
        raise TypeError("ArgTypeError missing required argument \'maintypes\' (pos 4)")


from Datapyck.errors import *


def Tellraw(selector="@a", text="", color="white", bold=False, italic=False, underline=False, strike=False,
            obfuscated=False):
    if type(selector) is not str:
        raise ArgTypeError("Tellraw", selector, 1, str)
    if type(color) is not str:
        raise ArgTypeError("Tellraw", color, 3, str)
    if type(bold) is not bool:
        raise ArgTypeError("Tellraw", bold, 4, bool)
    if type(italic) is not bool:
        raise ArgTypeError("Tellraw", italic, 5, bool)
    if type(underline) is not bool:
        raise ArgTypeError("Tellraw", underline, 6, bool)
    if type(strike) is not bool:
        raise ArgTypeError("Tellraw", strike, 7, bool)
    if type(obfuscated) is not bool:
        raise ArgTypeError("Tellraw", obfuscated, 8, bool)
    selector = selector.lower()
    if selector != "everyone" and not selector.startswith("@a") and selector != "random" and not selector.startswith(
            "@r") and selector != "nearest" and not selector.startswith(
            "@p") and selector != "self" and not selector.startswith("@s"):
        raise ValueError(f"unrecognized selector: \'{selector}\'")
    return "tellraw " + selector.replace("everyone", "@a").replace("random", "@r").replace("nearest", "@p").replace(
        "self", "@s") + " {\"text\":\"" + str(text) + "\", \"color\":\"" + color + "\", \"bold\":" + str(
        bold).lower() + ", \"italic\":" + str(italic).lower() + ", \"underlined\":" + str(
        underline).lower() + ", \"strikethrough\":" + str(strike).lower() + ", \"obfuscated\":" + str(
        obfuscated).lower() + "}"


def SelectorTellraw(selector="@a", textselector="@s", color="white", bold=False, italic=False, underline=False,
                    strike=False, obfuscated=False):
    if type(selector) is not str:
        raise ArgTypeError("SelectorTellraw", selector, 1, str)
    if type(textselector) is not str:
        raise ArgTypeError("SelectorTellraw", textselector, 2, str)
    if type(color) is not str:
        raise ArgTypeError("SelectorTellraw", color, 3, str)
    if type(bold) is not bool:
        raise ArgTypeError("SelectorTellraw", bold, 4, bool)
    if type(italic) is not bool:
        raise ArgTypeError("SelectorTellraw", italic, 5, bool)
    if type(underline) is not bool:
        raise ArgTypeError("SelectorTellraw", underline, 6, bool)
    if type(strike) is not bool:
        raise ArgTypeError("SelectorTellraw", strike, 7, bool)
    if type(obfuscated) is not bool:
        raise ArgTypeError("SelectorTellraw", obfuscated, 8, bool)
    selector = selector.lower()
    textselector = textselector.lower()
    if selector != "everyone" and not selector.startswith("@a") and selector != "random" and not selector.startswith(
            "@r") and selector != "nearest" and not selector.startswith(
            "@p") and selector != "self" and not selector.startswith("@s"):
        raise ValueError(f"unrecognized selector: \'{selector}\'")
    if textselector != "everyone" and not textselector.startswith(
            "@a") and textselector != "random" and not textselector.startswith(
            "@r") and textselector != "nearest" and not textselector.startswith(
            "@p") and textselector != "self" and not textselector.startswith("@s"):
        raise ValueError(f"unrecognized selector: \'{textselector}\'")
    return "tellraw " + selector.replace("everyone", "@a").replace("random", "@r").replace("nearest", "@p").replace(
        "self", "@s") + " {\"selector\":\"" + textselector + "\", \"color\":\"" + color + "\", \"bold\":" + str(
        bold).lower() + ", \"italic\":" + str(italic).lower() + ", \"underlined\":" + str(
        underline).lower() + ", \"strikethrough\":" + str(strike).lower() + ", \"obfuscated\":" + str(
        obfuscated).lower() + "}"


def KeybindTellraw(selector="@a", bind="forward", color="white", bold=False, italic=False, underline=False,
                   strike=False, obfuscated=False):
    if type(selector) is not str:
        raise ArgTypeError("KeybindTellraw", selector, 1, str)
    if type(bind) is not str:
        raise ArgTypeError("KeybindTellraw", bind, 2, str)
    if type(color) is not str:
        raise ArgTypeError("KeybindTellraw", color, 3, str)
    if type(bold) is not bool:
        raise ArgTypeError("KeybindTellraw", bold, 4, bool)
    if type(italic) is not bool:
        raise ArgTypeError("KeybindTellraw", italic, 5, bool)
    if type(underline) is not bool:
        raise ArgTypeError("KeybindTellraw", underline, 6, bool)
    if type(strike) is not bool:
        raise ArgTypeError("KeybindTellraw", strike, 7, bool)
    if type(obfuscated) is not bool:
        raise ArgTypeError("KeybindTellraw", obfuscated, 8, bool)
    selector = selector.lower()
    bind = bind.lower()
    if selector != "everyone" and not selector.startswith("@a") and selector != "random" and not selector.startswith(
            "@r") and selector != "nearest" and not selector.startswith(
            "@p") and selector != "self" and not selector.startswith("@s"):
        raise ValueError(f"unrecognized selector: \'{selector}\'")
    if bind != "forwards" and bind != "backwards" and bind != "left" and bind != "right" and bind != "jump" and bind != "sneak" and bind != "sprint" and bind != "inventory" and bind != "attack" and bind != "use" and bind != "pick" and bind != "drop" and bind != "swap" and bind != "slot1" and bind != "slot2" and bind != "slot3" and bind != "slot4" and bind != "slot5" and bind != "slot6" and bind != "slot7" and bind != "slot8" and bind != "slot9" and bind != "save" and bind != "load" and bind != "list" and bind != "chat" and bind != "command" and bind != "highlight" and bind != "screenshot" and bind != "cinematic" and bind != "fullscreen" and bind != "perspective":
        raise ValueError(f"unrecognized keybind: \'{bind}\'")
    return "tellraw " + selector.replace("everyone", "@a").replace("random", "@r").replace("nearest", "@p").replace(
        "self", "@s") + " {\"keybind\":\"" + bind.replace("forwards", "key.forward").replace("backwards",
                                                                                             "key.back").replace("left",
                                                                                                                 "key.left").replace(
        "right", "key.right").replace("jump", "key.jump").replace("sneak", "key.sneak").replace("sprint",
                                                                                                "key.sprint").replace(
        "inventory", "key.inventory").replace("attack", "key.attack").replace("use", "key.use").replace("pick",
                                                                                                        "key.pickItem").replace(
        "drop", "key.drop").replace("swap", "key.swapHands").replace("slot1", "key.hotbar.1").replace("slot2",
                                                                                                      "key.hotbar.2").replace(
        "slot3", "key.hotbar.3").replace("slot4", "key.hotbar.4").replace("slot5", "key.hotbar.5").replace("slot6",
                                                                                                           "key.hotbar.6").replace(
        "slot7", "key.hotbar.7").replace("slot8", "key.hotbar.8").replace("slot9", "key.hotbar.9").replace("save",
                                                                                                           "key.saveToolbarActivator").replace(
        "load", "key.loadToolbarActivator").replace("list", "key.playerlist").replace("chat", "key.chat").replace(
        "command", "key.command").replace("highlight", "key.spectatorOutines").replace("screenshot",
                                                                                       "key.screenshot").replace(
        "cinematic", "key.smoothCamera").replace("fullscreen", "key.fullscreen").replace("perspective",
                                                                                         "key.togglePerspective") + "\", \"color\":\"" + color + "\", \"bold\":" + str(
        bold).lower() + ", \"italic\":" + str(italic).lower() + ", \"underlined\":" + str(
        underline).lower() + ", \"strikethrough\":" + str(strike).lower() + ", \"obfuscated\":" + str(
        obfuscated).lower() + "}"


def CombineTellraw(selector, *tellraws):
    if type(selector) is not str:
        raise ArgTypeError("CombineTellraw", selector, 1, str)
    selector = selector.lower()
    if selector != "everyone" and not selector.startswith("@a") and selector != "random" and not selector.startswith(
            "@r") and selector != "nearest" and not selector.startswith(
            "@p") and selector != "self" and not selector.startswith("@s"):
        raise ValueError(f"unrecognized selector: \'{selector}\'")

    output = "tellraw " + selector.replace("everyone", "@a").replace("random", "@r").replace("nearest", "@p").replace(
        "self", "@s") + " [\"\""
    for tellraw in tellraws:
        for charnum, char in enumerate(tellraw):
            if char == "{":
                output += ", " + tellraw[charnum:]
                break
    return output + "]"